//
//  AppDelegate.m
//  CA Animation Capture
//
//  Created by Matt Long on 2/2/09.
//  Copyright 2009 Skye Road Systems, Inc.. All rights reserved.
//

#import "AppDelegate.h"


@implementation AppDelegate

- (void)awakeFromNib;
{
  [[window contentView] setWantsLayer:YES];

  layer = [CALayer layer];
  [layer setDelegate:self];
  [layer setBounds:CGRectMake(0, 0, 100, 100)];
  [layer setPosition:CGPointMake(74.0, 74.0)];
  [layer setBorderWidth:5.0];

  CGColorRef bgcolor = CGColorCreateGenericRGB(0.10, 0.10, 0.10, 0.50);
  [layer setBackgroundColor:bgcolor];  
  CFRelease(bgcolor);  
  
  CGColorRef bordercolor = CGColorCreateGenericRGB(0.10, 0.80, 0.10, 0.50);
  [layer setBorderColor:bordercolor];
  CFRelease(bordercolor);

  [[[window contentView] layer] addSublayer:layer];
  
}

- (IBAction)saveAnimation:(id)sender;
{
  NSSavePanel *savePanel;
  
  savePanel = [NSSavePanel savePanel];
  [savePanel setExtensionHidden:YES];
  [savePanel setCanSelectHiddenExtension:NO];
  [savePanel setTreatsFilePackagesAsDirectories:NO];
  
  if( [savePanel runModal] == NSOKButton )
  {
    movie = [[QTMovie alloc] initToWritableFile:[savePanel filename] error:nil];    
  }
  
  [self loadAnimations];

  timer = [NSTimer scheduledTimerWithTimeInterval:1.0/(NSTimeInterval)10.0
                                           target:self
                                         selector:@selector(updateTime:)
                                         userInfo:NULL
                                          repeats:YES];    
  
}

- (void)loadAnimations;
{
  CAAnimationGroup *group = [CAAnimationGroup animation];
  
  [group setAnimations:[NSArray arrayWithObjects:[self backgroundColorAnimation],
                          [self borderWidthAnimation],
                          [self positionAnimation],
                          [self boundsAnimation], nil]];
  
  [group setValue:@"mainGroup" forKey:@"name"];
  [group setDuration:5.0];
  [group setAutoreverses:YES];
  [group setDelegate:self];
  
  [layer addAnimation:group forKey:@"group"];
}

- (void)updateTime:(NSTimer*)theTimer;
{
  NSBitmapImageRep *image = [self getCurrentFrame];
  
  QTTime time = QTMakeTime(1, 10);
  NSDictionary *attrs = [NSDictionary dictionaryWithObject:@"png " forKey:QTAddImageCodecType];
  NSImage *img = [[NSImage alloc] initWithData:[image TIFFRepresentation]];
  [movie addImage:img forDuration:time withAttributes:attrs];
  
  [image release];
}

- (NSBitmapImageRep*)getCurrentFrame;
{
  CGContextRef    context = NULL;
  CGColorSpaceRef colorSpace;
  int bitmapByteCount;
  int bitmapBytesPerRow;
  
  int pixelsHigh = (int)[[[window contentView] layer] bounds].size.height;
  int pixelsWide = (int)[[[window contentView] layer] bounds].size.width;

  bitmapBytesPerRow   = (pixelsWide * 4);
  bitmapByteCount     = (bitmapBytesPerRow * pixelsHigh);
  
  colorSpace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
  
  context = CGBitmapContextCreate (NULL,
                                   pixelsWide,
                                   pixelsHigh,
                                   8,
                                   bitmapBytesPerRow,
                                   colorSpace,
                                   kCGImageAlphaPremultipliedLast);
  if (context== NULL)
  {
    NSLog(@"Failed to create context.");
    return nil;
  }
  
  CGColorSpaceRelease( colorSpace );
  
  [[[[window contentView] layer] presentationLayer] renderInContext:context];
  
  CGImageRef img = CGBitmapContextCreateImage(context);
  NSBitmapImageRep *bitmap = [[NSBitmapImageRep alloc] initWithCGImage:img];
  CFRelease(img);
  
  return bitmap;

}

- (void)animationDidStop:(CAAnimation *)theAnimation finished:(BOOL)flag;
{
  NSLog(@"Animation stopped: %@", theAnimation);
  
  id name = [theAnimation valueForKey:@"name"];
  if( name )
    if( [name isEqualToString:@"mainGroup"] )
    {
      [movie updateMovieFile];
      [timer invalidate];
    }
}


#pragma mark -
#pragma mark Animations

- (CAAnimation*)positionAnimation;
{
  CGMutablePathRef thePath = CGPathCreateMutable();
  CGPathMoveToPoint(thePath,NULL,74.0,74.0);
  CGPathAddCurveToPoint(thePath,NULL,74.0,500.0,
                        320.0,500.0,
                        320.0,74.0);

  CGPathAddCurveToPoint(thePath,NULL,320.0,500.0,
                        566.0,500.0,
                        566.0,74.0);
  
  CAKeyframeAnimation *animation = [CAKeyframeAnimation animationWithKeyPath:@"position"];
  
  [animation setPath:thePath];
  [animation setDuration:5.0];
  [animation setAutoreverses:YES];
  [animation setRotationMode:kCAAnimationRotateAuto];
  [animation setFillMode:kCAFillModeForwards];
  
  CFRelease(thePath);
  
  return animation;
}

- (CAAnimation*)backgroundColorAnimation;
{
	CAKeyframeAnimation *animation = [CAKeyframeAnimation animationWithKeyPath:@"backgroundColor"];
  
	[animation setDuration:5.0];

	NSMutableArray *values = [NSMutableArray array];
  
  CGColorRef red = CGColorCreateGenericRGB(1.0, 0.0, 0.0, 1.0);
  CGColorRef green = CGColorCreateGenericRGB(0.0, 1.0, 0.0, 1.0);
  CGColorRef blue = CGColorCreateGenericRGB(0.0, 0.0, 1.0, 1.0);
  
  [values addObject:(NSColor*)red];
  [values addObject:(NSColor*)green];
  [values addObject:(NSColor*)blue];

	[animation setValues:values];
  [animation setCalculationMode:kCAAnimationLinear];
  [animation setAutoreverses:YES];

	return animation;
}

- (CAAnimation*)borderWidthAnimation;
{
	CAKeyframeAnimation *animation = [CAKeyframeAnimation animationWithKeyPath:@"borderWidth"];
  
	[animation setDuration:5.0];
  
	NSMutableArray *values = [NSMutableArray array];
  
  [values addObject:[NSNumber numberWithFloat:3.25]];
  [values addObject:[NSNumber numberWithFloat:11.25]];
  [values addObject:[NSNumber numberWithFloat:40.25]];
  [values addObject:[NSNumber numberWithFloat:5.25]];
  [values addObject:[NSNumber numberWithFloat:18.0]];
  
	[animation setValues:values];
  [animation setAutoreverses:YES];

	return animation;
}

- (CAAnimation*)boundsAnimation;
{
	CAKeyframeAnimation *animation = [CAKeyframeAnimation animationWithKeyPath:@"bounds"];
  
	[animation setDuration:5.0]; 
  
	NSMutableArray *values = [NSMutableArray array];
  
  NSRect startRect = NSMakeRect(10.0, 10.0, 50.0, 50.0);
  NSRect nextRect = NSMakeRect(300.0, 300.0, 200.0, 200.0);
  NSRect lastRect = NSMakeRect(200.0, 200.0, 300.0, 300.0);
  NSRect r = NSMakeRect(10.0, 10.0, 50.0, 50.0);
    
  [values addObject:[NSValue valueWithRect:startRect]];
  [values addObject:[NSValue valueWithRect:nextRect]];
  [values addObject:[NSValue valueWithRect:lastRect]];
  [values addObject:[NSValue valueWithRect:r]];
  
	[animation setValues:values];
  [animation setAutoreverses:YES];
  
	return animation;
}


- (void)dealloc;
{
  [movie release], movie = nil;
  [super dealloc];
}

@end
